@@ -1,139 +1,8 @@
 # devtools::document("~/bin/umx"); devtools::install("~/bin/umx");
 # ========================================
-# = Not Typically used directly by users =
+# = Not used directly by users =
 # ========================================
 
-#' xmu_dot_make_residuals (not for end users)
-#'
-#'
-#' @param mxMat An A or S MxMatrix 
-#' @param latents optional list of latents to alter location of circles (defaults to NULL)
-#' @param showFixed Whether to show fixed values or not
-#' @param digits How many digits to report
-#' @param resid How to show residuals and variances default is "circle". Other option is "line"
-#' @return - list of variance names and variances
-#' @export
-#' @family xmu internal not for end user
-xmu_dot_make_residuals <- function(mxMat, latents = NULL, showFixed = TRUE, digits = 2, resid = c("circle", "line")) {
-	mxMat_vals   = mxMat$values
-	mxMat_free   = mxMat$free
-	mxMat_labels = mxMat$labels
-	mxMat_rows = dimnames(mxMat_free)[[1]]
-	mxMat_cols = dimnames(mxMat_free)[[2]]
-
-	variances = c()
-	varianceNames = c()
-	for(to in mxMat_rows ) { # rows
-		lowerVars  = mxMat_rows[1:match(to, mxMat_rows)]
-		for(from in lowerVars) { # columns
-			thisPathLabel = mxMat_labels[to, from]
-			thisPathFree  = mxMat_free[to, from]
-			thisPathVal   = round(mxMat_vals[to, from], digits)
-
-			if(thisPathFree){ prefix = "" } else { prefix = "@" }
-			# TODO currently all variances are labeled "a_with_a"
-			# Could diversify to "a_with_a", "var_a" & "resid_a"
-			if(thisPathFree | (thisPathVal !=0 && showFixed)) {
-				if((to == from)) {
-					if(resid =="circle"){
-						# TODO refactor based on mxGraphviz to support latents north
-						if(from %in% latents){
-							circleString = paste0(from, ' -> ', from, '[label="', prefix, thisPathVal, '", dir=both, headport=n, tailport=n]')
-						} else {
-							circleString = paste0(from, ' -> ', from, '[label="', prefix, thisPathVal, '", dir=both, headport=s, tailport=s]')
-						}
-						variances = append(variances, circleString)
-					} else if(resid =="line"){
-						varianceNames = append(varianceNames, paste0(from, '_var'))
-						variances = append(variances, paste0(from, '_var [label="', prefix, thisPathVal, '", shape = plaintext]'))
-					}					
-				}
-			}
-		}
-	}
-	return(list(varianceNames = varianceNames, variances = variances))
-}
-
-#' xmu_dot_make_paths (not for end users)
-#'
-#' Makes graphviz paths
-#'
-#' @param mxMat an MxMatrix
-#' @param stringIn input string
-#' @param heads 1 or 2
-#' @param showFixed to show fixed values or not
-#' @param comment a comment to include
-#' @param showResiduals Whether to show residuals
-#' @param pathLabels labels
-#' @param digits how many digits to report
-#' @return - string
-#' @export
-#' @family xmu internal not for end user
-xmu_dot_make_paths <- function(mxMat, stringIn, heads = NULL, showFixed = TRUE, comment = "More paths", showResiduals = TRUE, pathLabels = "labels", digits = 2) {
-	if(is.null(heads)){
-		stop("You must set 'heads' to 1 or 2 (was NULL)")
-	}
-	if(!heads %in% 1:2){
-		stop("You must set 'heads' to 1 or 2: was ", heads)
-	}
-	mxMat_vals   = mxMat@values
-	mxMat_free   = mxMat@free
-	mxMat_labels = mxMat@labels
-	mxMat_rows = dimnames(mxMat_free)[[1]]
-	mxMat_cols = dimnames(mxMat_free)[[2]]
-	if(!is.null(comment)){
-		stringIn = paste0(stringIn, "\n\t# ", comment, "\n")
-	}
-	if(heads == 1){
-		for(target in mxMat_rows ) {
-			for(source in mxMat_cols) {
-				thisPathLabel = mxMat_labels[target, source]
-				thisPathFree  = mxMat_free[target, source]
-				thisPathVal   = round(mxMat_vals[target, source], digits)
-
-				if(thisPathFree){ labelStart = ' [label="' } else { labelStart = ' [label="@' }
-
-				if(thisPathFree | ((showFixed & (thisPathVal != 0))) ) {
-					stringIn = paste0(stringIn, "\t", source, " -> ", target, labelStart, thisPathVal, '"];\n')
-				}else{
-					# print(paste0("thisPathFree = ", thisPathFree , "showFixed =", showFixed, "; thisPathVal = ", thisPathVal, "\n"))
-				}
-				
-			}
-		}
-	} else {
-		# heads = 2
-		for(target in mxMat_rows ) { # rows
-			lowerVars  = mxMat_rows[1:match(target, mxMat_rows)]
-			for(source in lowerVars) { # columns
-				thisPathLabel = mxMat_labels[target, source]
-				thisPathFree  = mxMat_free[target, source]
-				thisPathVal   = round(mxMat_vals[target, source], digits)
-
-				if(thisPathFree){ prefix = "" } else { prefix = "@" }
-
-				if(thisPathFree | ((showFixed & (thisPathVal != 0))) ) {
-					if(target == source) {
-						if(showResiduals){
-							stringIn = paste0(stringIn, "\t", source, "_var -> ", target, ";\n")
-						}
-					} else {
-						if(pathLabels == "both"){
-							stringIn = paste0(stringIn, "\t", source, " -> ", target, ' [dir=both, label="', thisPathLabel, "=", prefix, thisPathVal, "\"];\n")
-						} else if(pathLabels == "labels"){
-							stringIn = paste0(stringIn, "\t", source, " -> ", target, ' [dir=both, label="', thisPathLabel, "\"];\n")
-						}else {
-							# pathLabels = "none"
-							stringIn = paste0(stringIn, "\t", source, " -> ", target, ' [dir=both, label="', prefix, thisPathVal, "\"];\n")
-						}
-					}
-				}
-			}
-		}
-	}
-	return(stringIn)
-}
-
 #' xmuLabel_MATRIX_Model (not a user function)
 #'
 #' This function will label all the free parameters in a (non-RAM) OpenMx \code{\link{mxModel}}
@@ -243,8 +112,8 @@ xmuLabel_RAM_Model <- function(model, suffix = "", labelFixedCells = TRUE, overR
 	# = Add means labels if needed =
 	# ==============================
 	# TODO add a test case with raw data but no means...
-	if(!is.null(model@data)){
-		if(model@data@type == "raw" & is.null(model$M)) {
+	if(!is.null(model$data)){
+		if(model$data$type == "raw" & is.null(model$M)) {
 			message("You are using raw data, but have not yet added paths for the means\n")
 			message("Do this with umxPath(means = 'var')")
 		}
@@ -345,13 +214,13 @@ xmuLabel_Matrix <- function(mx_matrix = NA, baseName = NA, setfree = FALSE, drop
 		newLabels[upper.tri(newLabels, diag = FALSE)] <- mirrorLabels[upper.tri(mirrorLabels, diag = FALSE)]
 		diag(newLabels) <- NA
 	} else if(type == "IdenMatrix" | type == "UnitMatrix" | type == "ZeroMatrix") {
-		# message("umxLabel Ignored ", type, " matrix ", mx_matrix@name, " - it has no free values!")
+		# message("umxLabel Ignored ", type, " matrix ", mx_matrix$name, " - it has no free values!")
 		return(mx_matrix)
 	} else {
 		return(paste0("You tried to set type ", "to ", omxQuotes(type)));
 	}
 	# Set labels
-	mx_matrix@labels <- newLabels;
+	mx_matrix$labels <- newLabels;
 	if(setfree == FALSE) {
 		# return("Matrix Specification not used: leave free as set in mx_matrix") 
 	} else {
@@ -363,16 +232,16 @@ xmuLabel_Matrix <- function(mx_matrix = NA, baseName = NA, setfree = FALSE, drop
 			newLabels[lower.tri(newLabels, diag = FALSE)] -> lower.labels;
 			newLabels[upper.tri(newLabels, diag = FALSE)] <- lower.labels;
 		} else {
-			mx_matrix@free <- newFree
+			mx_matrix$free <- newFree
 		}
 		# newFree[is.na(newLabels)]=NA; # (validated by mxMatrix???)
 	}
 	if(!is.na(jiggle)){
-		mx_matrix@values <- umxJiggle(mx_matrix$values, mean = 0, sd = jiggle, dontTouch = drop) # Expecting sd
+		mx_matrix$values <- umxJiggle(mx_matrix$values, mean = 0, sd = jiggle, dontTouch = drop) # Expecting sd
 	}
 	# TODO this might want something to equate values after jiggling around equal labels?
 	if(!is.na(boundDiag)){
-		diag(mx_matrix@lbound) <- boundDiag # bound diagonal to be positive 
+		diag(mx_matrix$lbound) <- boundDiag # bound diagonal to be positive 
 	}
 	return(mx_matrix)
 }
@@ -466,25 +335,25 @@ xmuMakeDeviationThresholdsMatrices <- function(df, droplevels, verbose) {
 	initialLowerLim  = -1
 	initialUpperLim  =  1
 	# Fill first row of deviations_for_thresh with useful lower thresholds, perhaps -1 or .5 SD (nthresh/2)
-	deviations_for_thresh@free[1,]   <- TRUE
-	deviations_for_thresh@values[1,] <- initialLowerLim # Start with an even -2. Might spread this a bit for different levels, or centre on 0 for 1 threshold
-	deviations_for_thresh@labels[1,] <- paste("ThreshBaseline1", 1:nOrdinal, sep ="_")
-	deviations_for_thresh@lbound[1,] <- -7 # baseline limit in SDs
-	deviations_for_thresh@ubound[1,] <-  7 # baseline limit in SDs
+	deviations_for_thresh$free[1,]   <- TRUE
+	deviations_for_thresh$values[1,] <- initialLowerLim # Start with an even -2. Might spread this a bit for different levels, or centre on 0 for 1 threshold
+	deviations_for_thresh$labels[1,] <- paste("ThreshBaseline1", 1:nOrdinal, sep ="_")
+	deviations_for_thresh$lbound[1,] <- -7 # baseline limit in SDs
+	deviations_for_thresh$ubound[1,] <-  7 # baseline limit in SDs
 
 	for(n in 1:nOrdinal){
 		thisThreshMinus1 = levelList[n] -1
 		stepSize = (initialUpperLim-initialLowerLim)/thisThreshMinus1
-		deviations_for_thresh@values[2:thisThreshMinus1,n] = (initialUpperLim - initialLowerLim) / thisThreshMinus1
-		deviations_for_thresh@labels[2:thisThreshMinus1,n] = paste("ThreshDeviation", 2:thisThreshMinus1, n, sep = "_")
-		deviations_for_thresh@free  [2:thisThreshMinus1,n] = TRUE
-		deviations_for_thresh@lbound[2:thisThreshMinus1,n] = .001
+		deviations_for_thresh$values[2:thisThreshMinus1,n] = (initialUpperLim - initialLowerLim) / thisThreshMinus1
+		deviations_for_thresh$labels[2:thisThreshMinus1,n] = paste("ThreshDeviation", 2:thisThreshMinus1, n, sep = "_")
+		deviations_for_thresh$free  [2:thisThreshMinus1,n] = TRUE
+		deviations_for_thresh$lbound[2:thisThreshMinus1,n] = .001
 		if(thisThreshMinus1 < maxThreshMinus1) {
 			# pad the shorter var's excess rows with fixed@99 so humans can see them...
-			deviations_for_thresh@values[(thisThreshMinus1+1):maxThreshMinus1,n] <- (-99)
+			deviations_for_thresh$values[(thisThreshMinus1+1):maxThreshMinus1,n] <- (-99)
 
-			deviations_for_thresh@labels[(thisThreshMinus1+1):maxThreshMinus1,n] <- paste("unusedThresh", min(thisThreshMinus1 + 1, maxThreshMinus1), n, sep = "_")
-			deviations_for_thresh@free  [(thisThreshMinus1+1):maxThreshMinus1,n] <- F
+			deviations_for_thresh$labels[(thisThreshMinus1+1):maxThreshMinus1,n] <- paste("unusedThresh", min(thisThreshMinus1 + 1, maxThreshMinus1), n, sep = "_")
+			deviations_for_thresh$free  [(thisThreshMinus1+1):maxThreshMinus1,n] <- F
 		}
 	}
 
@@ -551,8 +420,8 @@ xmu_start_value_list <- function(mean = 1, sd = NA, n = 1) {
 #' )
 #' m1 = umx:::xmuPropagateLabels(m1, suffix = "MZ")
 xmuPropagateLabels <- function(model, suffix = "", verbose = TRUE) {
-	model@matrices <- lapply(model$matrices , xmuLabel_Matrix   , suffix = suffix, verbose = verbose)
-	model@submodels <- lapply(model@submodels, xmuPropagateLabels, suffix = suffix, verbose = verbose)
+	model@matrices  <- lapply(model$matrices , xmuLabel_Matrix   , suffix = suffix, verbose = verbose)
+	model@submodels <- lapply(model$submodels, xmuPropagateLabels, suffix = suffix, verbose = verbose)
 	return(model)
 }
 
@@ -601,17 +470,17 @@ xmuMI <- function(model, vector = TRUE) {
 		}
 		res
 	}
-	A <- model$A@values
-	P <- model$S@values
-	S <- model$data@observed
-	J <- model$F@values
+	A <- model$A$values
+	P <- model$S$values
+	S <- model$data$observed
+	J <- model$F$values
 	m <- dim(A)[1]
-	which.free <- c(model$A@free, model$S@free & upper.tri(diag(m), diag= TRUE))
+	which.free <- c(model$A$free, model$S$free & upper.tri(diag(m), diag= TRUE))
 	vars       <- colnames(A)
-	parNames   <- c(model$A@labels, model$S@labels)
+	parNames   <- c(model$A$labels, model$S$labels)
 	parNames[is.na(parNames)] <- c(outer(vars, vars, paste, sep=' <- '),
 	outer(vars, vars, paste, sep=' <-> '))[is.na(parNames)]
-	NM     <- model$data@numObs - 1
+	NM     <- model$data$numObs - 1
 	I.Ainv <- solve(diag(m) - A) 
 	C      <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)
 	Cinv   <- solve(C)    
@@ -652,7 +521,7 @@ xmuMI <- function(model, vector = TRUE) {
 	names(mod.indices) <- parNames
 	names(par.change)  <- parNames
 	if (vector) {
-		which.ret <- c(!model$A@free & !diag(m), !model$S@free) # & upper.tri(diag(m), diag= TRUE))
+		which.ret <- c(!model$A$free & !diag(m), !model$S$free) # & upper.tri(diag(m), diag= TRUE))
 		sel <- order(mod.indices[which.ret], decreasing= TRUE)
 		ret <- list(mi=mod.indices[which.ret][sel], par.change=par.change[which.ret][sel])
 	} else {
@@ -664,12 +533,12 @@ xmuMI <- function(model, vector = TRUE) {
 		rownames(mod.P) <- colnames(mod.P) <- vars
 		rownames(par.A) <- colnames(par.A) <- vars
 		rownames(par.P) <- colnames(par.P) <- vars
-		mod.A[model$A@free] <- NA
-		par.A[model$A@free] <- NA
+		mod.A[model$A$free] <- NA
+		par.A[model$A$free] <- NA
 		diag(mod.A) <- NA
 		diag(par.A) <- NA
-		mod.P[model$S@free] <- NA
-		par.P[model$S@free] <- NA
+		mod.P[model$S$free] <- NA
+		par.P[model$S$free] <- NA
 		ret <- list(mod.A=mod.A, par.A=par.A, mod.S=mod.P, par.S=par.P)
 	}
     utils::setTxtProgressBar(bar, 5)
@@ -798,24 +667,160 @@ xmuMakeOneHeadedPathsFromPathList <- function(sourceList, destinationList) {
 	return(toAdd)
 }
 
-xmu_dot_maker <- function(model, dotFilename, digraph){
-	if(!is.na(dotFilename)){
-		if(dotFilename == "name"){
-			dotFilename = paste0(model@name, ".dot")
+xmu_dot_maker <- function(model, file, digraph){
+	if(!is.na(file)){
+		if(file == "name"){
+			file = paste0(model$name, ".gv")
 		}
-		cat(digraph, file = dotFilename) # write to file
-		if(umx_check_OS("OSX")){
-			umx_open(dotFilename);
-		} else if(umx_check_OS("Windows")){
-			shell(paste0("dot -Tpdf -O ", shQuote(dotFilename)), "cmd.exe");
-			umx_open(paste0(dotFilename, ".pdf"))
+		cat(digraph, file = file) # write to file
+		if(umx_set_plot_format()=="DiagrammeR"){			
+			DiagrammeR(diagram = "~tim/base.gv", type = "grViz")
+			DiagrammeR(diagram = file, type = "grViz")
 		} else {
-			system(paste0("dot -Tpdf -O ", shQuote(dotFilename)));
-			umx_open(paste0(dotFilename, ".pdf"))
+			if(umx_check_OS("OSX")){
+				umx_open(file);
+			} else if(umx_check_OS("Windows")){
+				shell(paste0("dot -Tpdf -O ", shQuote(file)), "cmd.exe");
+				umx_open(paste0(file, ".pdf"))
+			} else {
+				system(paste0("dot -Tpdf -O ", shQuote(file)));
+				umx_open(paste0(file, ".pdf"))
+			}
+			# dot -Tpdf -O yourFilename.gv
+			# creates "yourFilename.gv.pdf"
 		}
-		# dot -Tpdf -O yourFilename.dot
-		# creates "yourFilename.dot.pdf"
 	} else {
 		return (cat(digraph));
 	}
 }
+
+#' xmu_dot_make_residuals (not for end users)
+#'
+#'
+#' @param mxMat An A or S mxMatrix 
+#' @param latents Optional list of latents to alter location of circles (defaults to NULL)
+#' @param showFixed Whether to show fixed values or not
+#' @param digits How many digits to report
+#' @param resid How to show residuals and variances default is "circle". Other option is "line"
+#' @return - list of variance names and variances
+#' @export
+#' @family xmu internal not for end user
+xmu_dot_make_residuals <- function(mxMat, latents = NULL, showFixed = TRUE, digits = 2, resid = c("circle", "line")) {
+	mxMat_vals   = mxMat$values
+	mxMat_free   = mxMat$free
+	mxMat_labels = mxMat$labels
+	mxMat_rows = dimnames(mxMat_free)[[1]]
+	mxMat_cols = dimnames(mxMat_free)[[2]]
+
+	variances = c()
+	varianceNames = c()
+	for(to in mxMat_rows ) { # rows
+		lowerVars  = mxMat_rows[1:match(to, mxMat_rows)]
+		for(from in lowerVars) { # columns
+			thisPathLabel = mxMat_labels[to, from]
+			thisPathFree  = mxMat_free[to, from]
+			thisPathVal   = round(mxMat_vals[to, from], digits)
+
+			if(thisPathFree){ prefix = "" } else { prefix = "@" }
+			# TODO currently all variances are labeled "a_with_a"
+			# Could diversify to "a_with_a", "var_a" & "resid_a"
+			if(thisPathFree | (thisPathVal !=0 && showFixed)) {
+				if((to == from)) {
+					if(resid =="circle"){
+						# TODO support latents north (see mxGraphviz)
+						if(from %in% latents){
+							circleString = paste0(from, ' -> ', from, '[label="', prefix, thisPathVal, '", dir=both, headport=n, tailport=n]')
+						} else {
+							circleString = paste0(from, ' -> ', from, '[label="', prefix, thisPathVal, '", dir=both, headport=s, tailport=s]')
+						}
+						variances = append(variances, circleString)
+					} else if(resid =="line"){
+						varianceNames = append(varianceNames, paste0(from, '_var'))
+						variances = append(variances, paste0(from, '_var [label="', prefix, thisPathVal, '", shape = plaintext]'))
+					}					
+				}
+			}
+		}
+	}
+	return(list(varianceNames = varianceNames, variances = variances))
+}
+
+#' xmu_dot_make_paths (not for end users)
+#'
+#' Makes graphviz paths
+#'
+#' @param mxMat An mxMatrix
+#' @param stringIn Input string
+#' @param heads 1 or 2 arrows (default NULL - you must set this)
+#' @param showFixed Whether show fixed values or not (defaults to TRUE)
+#' @param comment A comment to include
+#' @param showResiduals Whether to show residuals
+#' @param pathLabels labels
+#' @param digits how many digits to report
+#' @return - string
+#' @export
+#' @family xmu internal not for end user
+xmu_dot_make_paths <- function(mxMat, stringIn, heads = NULL, showFixed = TRUE, comment = "More paths", showResiduals = TRUE, pathLabels = "labels", digits = 2) {
+	if(is.null(heads)){
+		stop("You must set 'heads' to 1 or 2 (was NULL)")
+	}
+	if(!heads %in% 1:2){
+		stop("You must set 'heads' to 1 or 2: was ", heads)
+	}
+	mxMat_vals   = mxMat$values
+	mxMat_free   = mxMat$free
+	mxMat_labels = mxMat$labels
+	mxMat_rows = dimnames(mxMat_free)[[1]]
+	mxMat_cols = dimnames(mxMat_free)[[2]]
+	if(!is.null(comment)){
+		stringIn = paste0(stringIn, "\n\t# ", comment, "\n")
+	}
+	if(heads == 1){
+		for(target in mxMat_rows ) {
+			for(source in mxMat_cols) {
+				thisPathLabel = mxMat_labels[target, source]
+				thisPathFree  = mxMat_free[target, source]
+				thisPathVal   = round(mxMat_vals[target, source], digits)
+
+				if(thisPathFree){ labelStart = ' [label="' } else { labelStart = ' [label="@' }
+
+				if(thisPathFree | ((showFixed & (thisPathVal != 0))) ) {
+					stringIn = paste0(stringIn, "\t", source, " -> ", target, labelStart, thisPathVal, '"];\n')
+				}else{
+					# print(paste0("thisPathFree = ", thisPathFree , "showFixed =", showFixed, "; thisPathVal = ", thisPathVal, "\n"))
+				}
+				
+			}
+		}
+	} else {
+		# heads = 2
+		for(target in mxMat_rows ) { # rows
+			lowerVars  = mxMat_rows[1:match(target, mxMat_rows)]
+			for(source in lowerVars) { # columns
+				thisPathLabel = mxMat_labels[target, source]
+				thisPathFree  = mxMat_free[target, source]
+				thisPathVal   = round(mxMat_vals[target, source], digits)
+
+				if(thisPathFree){ prefix = "" } else { prefix = "@" }
+
+				if(thisPathFree | ((showFixed & (thisPathVal != 0))) ) {
+					if(target == source) {
+						if(showResiduals){
+							stringIn = paste0(stringIn, "\t", source, "_var -> ", target, ";\n")
+						}
+					} else {
+						if(pathLabels == "both"){
+							stringIn = paste0(stringIn, "\t", source, " -> ", target, ' [dir=both, label="', thisPathLabel, "=", prefix, thisPathVal, "\"];\n")
+						} else if(pathLabels == "labels"){
+							stringIn = paste0(stringIn, "\t", source, " -> ", target, ' [dir=both, label="', thisPathLabel, "\"];\n")
+						}else {
+							# pathLabels = "none"
+							stringIn = paste0(stringIn, "\t", source, " -> ", target, ' [dir=both, label="', prefix, thisPathVal, "\"];\n")
+						}
+					}
+				}
+			}
+		}
+	}
+	return(stringIn)
+}
\ No newline at end of file
